//Validate Field Entry v1.2

//Purpose:
//Verifies that specified additional fields have a value entered
//Prevents saving the event if the specified additional field's value is empty

//Action Type: Before Event Save
//Prevent Default Action: Yes

//More info on Before Event Save actions and objects here:
//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#BeforeSave

validateFieldEntry();

function validateFieldEntry() {
	'use strict';

	var listOfFieldsToValidate = [];

	//!EDIT HERE!
	//Adds the names of the fields to the list to be validated
	//Replace 'MyField' with your mapped field name
	//Duplicate this line for each additional field you would like to validate
	listOfFieldsToValidate.push('MyField');

 //----------- You shouldn't need to edit below this line -------------------
	var emptyFields = '';
	var emptyField;

	//Check each field to validate for an empty value
	listOfFieldsToValidate.forEach(function (currentField) {
		emptyField = validateField(editEvent, currentField);
		if (emptyField !== false) {
			emptyFields += (emptyFields === '') ? emptyField : ', ' + emptyField;
		}
	});

	//Save the event if all the fields have values in them
	if (emptyFields === '') {

		//returning true so our action does not prevent the default behavior (save)
		return true;
	}

	//Show alert and prompt to save or revert
	//More info on the showModal function here:
	//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#utilities.showModal
	utilities.showModal('Additional field value is empty',
	 'The following fields have empty values: ' +
		emptyFields + '.  Would you like to continue saving the event?',
		'Cancel', function () {

			//More info on the action object here:
			//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#action
			action.callbacks.cancel();
		}, 'Save',	function () {

			//More info on the action object here:
			//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#action
			action.callbacks.confirm();
		});
}



//Function to validate an event field. Returns the field label if the value is empty
function validateField(editEvent, fieldName) {
	var fieldID = getFieldIDByMappedName(editEvent, fieldName);
	if (fieldID) {
		var fieldValue = editEvent[fieldID];
		if (!fieldValue || fieldValue === '' ||
		 (Array.isArray(fieldValue) && fieldValue.length === 0)) {
			return getFieldDisplayName(editEvent, fieldID);
		}
	}
	return false;
}

//Returns the ID of the field from its mapped field name
function getFieldIDByMappedName(event, fieldName) {

  //More info on the event.schedule object here:
	//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#event.schedule
	if (event.schedule.fieldMap) {
		for (var field in event.schedule.fieldMap) {
			if (event.schedule.fieldMap[field] === fieldName) {
				return field;
			}
		}
		return null;
	}
  else {
		return null;
	}
}

//Returns the display name of the specified field
function getFieldDisplayName(event, fieldID) {
	var displayName = fieldID;

	//More info on the event.schedule object here:
	//https://www.seedcode.com/pmwiki/index.php?n=DayBackOnline.CustomActionObjects#event.schedule
	if (event.schedule.customFields && event.schedule.customFields[fieldID]){
		displayName = event.schedule.customFields[fieldID].name;
	}
	else if (event.schedule.labelMap && event.schedule.labelMap[fieldID]) {
		displayName = event.schedule.labelMap[fieldID];
	}

	return displayName;
}
